#ifdef GL_ES
    #define LOWP lowp
    precision mediump float;
#else
    #define LOWP
#endif

// regular data we always get
uniform sampler2D u_texture;
varying LOWP vec4 v_color;
varying vec2 v_texCoord;

// the mode
varying float v_mode;

// we set them at halfs for precision. For example passthrough is 0.f, so we check if mode < 0.5.
#define PASSTHROUGH 0.5
#define PLAIN_COLOR 1.5
#define OUTLINE 2.5

// ################# UNIFORMS FOR OUTLINE SHADER #######################
// The inverse of the viewport dimensions along X and Y
uniform vec2 v_viewportInverse;
// Color of the outline
uniform vec4 v_outlineColor;
// Alpha of the model
uniform float u_alpha;
// Thickness of the outline
uniform float v_outlineThickness;
// Step to check for neighbors
uniform float v_outlineStep;
#define ALPHA_VALUE_BORDER 0.5



void main() {
    if (v_mode < PASSTHROUGH) {
        gl_FragColor = v_color * texture2D(u_texture, v_texCoord);
    }
    else if (v_mode < PLAIN_COLOR) {
        gl_FragColor = v_color * texture2D(u_texture, v_texCoord).a;
    }
    else if (v_mode < OUTLINE) {
        vec2 T = v_texCoord.xy;

        float alpha = 0.0;
        bool allin = true;
        if (texture2D(u_texture, T).a != 1.0) {
            for( float ix = -v_outlineThickness; ix <= v_outlineThickness; ix += v_outlineStep )
            {
                for( float iy = -v_outlineThickness; iy <= v_outlineThickness; iy += v_outlineStep )
                {
                    float newAlpha = texture2D(u_texture, T + vec2(ix, iy) * v_viewportInverse).a;
                    allin = allin && newAlpha > ALPHA_VALUE_BORDER;
                    if (newAlpha > ALPHA_VALUE_BORDER && newAlpha >= alpha)
                    {
                        alpha = newAlpha;
                    }
                }
           }
        }
        if (allin)
        {

            gl_FragColor = texture2D(u_texture, T).rgba;
        }
        else {
            gl_FragColor = vec4(v_outlineColor.rgb,alpha);
        }
    }
}
